/*
 * @(#)JPassGenConsole.java  2002-08-12
 *
 * Version 1.04
 *
 * Copyright (C) 26 juillet 2002 hughes monget
 * hmonget@users.sourceforge.net
 * http://jpassgen.sourceforge.net/
 *
 * This program is free software; you can rediargibute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * This product includes software developed by the Apache Software Foundation
 * (http://www.apache.org/)
 */

package net.sourceforge.jpassgen;

import java.util.List;

import org.apache.avalon.excalibur.cli.CLArgsParser;
import org.apache.avalon.excalibur.cli.CLOption;
import org.apache.avalon.excalibur.cli.CLOptionDescriptor;
import org.apache.avalon.excalibur.cli.CLUtil;

/**
 * This class is the main application of the password generator project:
 * It is a <b>console</b> (text based) version.
 * <br>
 * This product includes software developed by the Apache Software Foundation.
 * (<a href="http://www.apache.org/"
 * target="_blank">http://www.apache.org/</a>)
 *
 * @author hughes monget 
 */
public class JPassGenConsole {

    //////////////////
    // CLASS FIELDS //
    //////////////////
    
    /**
     * The default length of the passwords to generate.
     */ 
    public static final int DEFAULT_LENGTH = 32;

    /**
     * The default count of the passwords to generate.
     */ 
    public static final int DEFAULT_COUNT  = 1;

    /**
     * help option.
     */
    private static final int HELP_OPT   = 'h';

    /**
     * Lowercase characters option.
     */
    private static final int LOWERS_OPT = 'w';

    /**
     * Uppercase characters option.
     */
    private static final int UPPERS_OPT = 'u';

    /**
     * digit characters option.
     */
    private static final int DIGITS_OPT = 'd';

    /**
     * Other characters option.
     */
    private static final int OTHERS_OPT = 't';

    /**
     * Alls characters option.
     */
    private static final int ALLS_OPT   = 'a';

    /**
     * Password count option.
     */
    private static final int COUNT_OPT  = 'c';

    /**
     * Password length option.
     */
    private static final int LENGTH_OPT = 'l';

    /**
     * Options descriptor.
     */ 
    private static final CLOptionDescriptor[] OPTIONS =

            new CLOptionDescriptor[] {

        new CLOptionDescriptor("help",
                CLOptionDescriptor.ARGUMENT_DISALLOWED 
                | CLOptionDescriptor.DUPLICATES_ALLOWED,
                JPassGenConsole.HELP_OPT, "print this help"),

        new CLOptionDescriptor("lowers",
                CLOptionDescriptor.ARGUMENT_DISALLOWED 
                | CLOptionDescriptor.DUPLICATES_ALLOWED,
                JPassGenConsole.LOWERS_OPT, "lowercase characters"),

        new CLOptionDescriptor("uppers",
                CLOptionDescriptor.ARGUMENT_DISALLOWED 
                | CLOptionDescriptor.DUPLICATES_ALLOWED,
                JPassGenConsole.UPPERS_OPT, "uppercase characters"),

        new CLOptionDescriptor("digits", 
                CLOptionDescriptor.ARGUMENT_DISALLOWED 
                | CLOptionDescriptor.DUPLICATES_ALLOWED,
                JPassGenConsole.DIGITS_OPT, "digit characters"),

        new CLOptionDescriptor("others",
                CLOptionDescriptor.ARGUMENT_DISALLOWED 
                | CLOptionDescriptor.DUPLICATES_ALLOWED,
                JPassGenConsole.OTHERS_OPT, "other characters"),

        new CLOptionDescriptor("alls",  
                CLOptionDescriptor.ARGUMENT_DISALLOWED 
                | CLOptionDescriptor.DUPLICATES_ALLOWED,
                JPassGenConsole.ALLS_OPT, "alls characters"),

        new CLOptionDescriptor("count", CLOptionDescriptor.ARGUMENT_REQUIRED,
                JPassGenConsole.COUNT_OPT, "set password count"),

        new CLOptionDescriptor("length", CLOptionDescriptor.ARGUMENT_REQUIRED,
                JPassGenConsole.LENGTH_OPT, "set password length")

    };

    ///////////////////
    // CLASS METHODS //
    ///////////////////

    /**
    * Main function. 
    * @param args command line's arguments. 
    */ 
    public static void main(String args[]) {
        new JPassGenConsole(args);
    }

    /////////////////////
    // INSTANCE FIELDS //
    /////////////////////

    /**
     * The mask of characters for the <code>RandomPassword()</code> object.
     */
    private int _mask = 0;

    /**
     * The current length of the passwords to generate.
     */
    private int _passwordCount  = JPassGenConsole.DEFAULT_COUNT;

    /**
     * The current length of the passwords to generate.
     */
    private int _passwordLength = JPassGenConsole.DEFAULT_LENGTH;

    //////////////////
    // CONSTRUCTORS //
    //////////////////

    /** 
     * Constructor: we parse the command line.
     *
     * @param args command line's arguments. 
     */
    JPassGenConsole(String args[]) {

        boolean lowers = false;
        boolean uppers = false;
        boolean digits = false;
        boolean others = false;
        boolean alls   = false;

        // Parse the arguments
        final CLArgsParser parser = new CLArgsParser(args, OPTIONS);

        // Make sure that there was no errors parsing arguments
        if (null != parser.getErrorString()) {
            printErrorAndExit("Error: " + parser.getErrorString());
        }

        // Get a list of parsed options
        final List options = parser.getArguments();
        final int size = options.size();

        // Set options.
        for (int ii = 0 ; ii < size ; ii++) {

            final CLOption option = (CLOption) options.get(ii);

            switch (option.getId()) {

                case CLOption.TEXT_ARGUMENT:
                    //This occurs when a user supplies an argument that
                    //is not an option
                    this.printErrorAndExit("Unknown arg: " 
                            + option.getArgument());
                    break;

                case JPassGenConsole.HELP_OPT:
                    // We display the help.
                    this.printHelp();
                    System.exit(0);
                    break;

                case JPassGenConsole.LOWERS_OPT:
                    lowers = true;
                    break;

                case JPassGenConsole.UPPERS_OPT:
                    uppers = true;
                    break;

                case JPassGenConsole.DIGITS_OPT:
                    digits = true;
                    break;

                case JPassGenConsole.OTHERS_OPT:
                    others = true;
                    break;

                case JPassGenConsole.ALLS_OPT:
                    alls = true;
                    break;

                case JPassGenConsole.COUNT_OPT: {
                    // We check the argument.
                    String msg = "Invalid argument to option -c";
                    try {
                        _passwordCount =
                                Integer.parseInt(option.getArgument(0));
                    } catch (NumberFormatException e) {
                        this.printErrorAndExit(msg);
                    }
                    if (_passwordCount <= 0) {
                        this.printErrorAndExit(msg);
                    }
                    break;
                }

                case JPassGenConsole.LENGTH_OPT: {
                    // We check the argument.
                    String msg = "Invalid argument to option -l";
                    try {
                        _passwordLength =
                                Integer.parseInt(option.getArgument(0));
                    } catch (NumberFormatException e) {
                        this.printErrorAndExit(msg);
                    }
                    if (_passwordLength <= 0) {
                        this.printErrorAndExit(msg);
                    }
                    break;
                }
            } // end switch
        } // end for

        // We construct the mask.
        if (alls) {
            _mask = RandomPassword.ALLS;
        } else {
            if (lowers) {
                _mask += RandomPassword.LOWERS;
            }
            if (uppers) {
                _mask += RandomPassword.UPPERS;
            }
            if (digits) {
                _mask += RandomPassword.DIGITS;
            }
            if (others) {
                _mask += RandomPassword.OTHERS;
            }
        }

        // We test the mask.
        if (_mask == 0) {
            this.printErrorAndExit("Error: No characters found");
        } else {
            // All is ok, we can generate passwords.
            this.printPassword();
            System.exit(0);
        }
    }

    //////////////////////
    // INSTANCE METHODS //
    //////////////////////

    /**
     * We create and display genererated passwords.
     */
    void printPassword() {

        RandomPassword rp = new RandomPassword(_passwordLength, _mask);
        StringBuffer buf = new StringBuffer();
        for (int ii = 0 ; ii < _passwordCount ; ii++) {
            buf.append(rp.getNext());
            buf.append("\n");
        }
        System.out.print(buf);
        buf = null;
    }

    /**
     * We display the help.
     */
    void printHelp() {

        StringBuffer buf = new StringBuffer();
        buf.append("JPassGen v1.04 (GPL 2002)");
        buf.append(" is a secure and strong password generator.");
        buf.append("\n");
        buf.append("(http://jpassgen.sourceforge.net/)");
        buf.append("\n");
        buf.append("This product includes software");
        buf.append(" developed by the Apache Software Foundation.");
        buf.append("\n");
        buf.append("(http://www.apache.org/)");
        buf.append("\n");
        buf.append("Options:");
        buf.append("\n");
        buf.append(CLUtil.describeOptions(OPTIONS));
        System.out.print(buf);
        buf = null;
    }

    /**
     * We display an error message and exit.
     *
     * @param msg The error message.
     */ 
    void printErrorAndExit(String msg) {

        System.err.print(msg);
        System.exit(1);
    }

} // End of JPassGenConsole.java
