/*
 * @(#)Parameters.java  2002-12-26
 *
 * Version 1.04
 *
 * Copyright (C) 26 juillet 2002 hughes monget
 * hmonget@users.sourceforge.net
 * http://jpassgen.sourceforge.net/
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package net.sourceforge.jpassgen;

import java.util.MissingResourceException;
import java.util.ResourceBundle;

/**
 * This class provides the parameters of the application.
 *
 * @author hughes monget
 */
class Parameters {

    //////////////////
    // CLASS FIELDS //
    //////////////////

    /**
     * The default count of psswords.
     */
    private static final int DEFAULT_COUNT = 10;

    /**
     * the default maximum length
     */
    private static final int DEFAULT_MAX_LENGTH = 32;

    /**
     * the default minimum length
     */
    private static final int DEFAULT_MIN_LENGTH = 4;

    /**
     * The base name of the config file. 
     */
    private static final String IDS_FILENAME = "config";

    /**
     * The singleton instance.
     */
    private static Parameters singleton = null;


    /**
     * The resource bundle.
     */
    private ResourceBundle _rb = null;

    ///////////////////
    // CLASS METHODS //
    ///////////////////

    /**
     * The "getInstance()" method of "Singleton pattern"
     * @return the strings server singleton. 
     */
    static synchronized Parameters getInstance() {

        if (Parameters.singleton == null) {
            Parameters.singleton = new Parameters();
        }
        return Parameters.singleton;
    }

    //////////////////
    // CONSTRUCTORS //
    //////////////////

    /** 
     * The default constructor is private; we initialise the resource bundle.
     */
    private Parameters() {

        String path = getClass().getPackage().getName() + "." + IDS_FILENAME;

        try {
            _rb = ResourceBundle.getBundle(path);
        } catch (MissingResourceException e) {
             System.err.println("** MissingResourceException:" 
                     + e.getMessage());
        }
    }

    //////////////////////
    // INSTANCE METHODS //
    //////////////////////

    /**
     * We return the count of passwords.
     * @return the count of passwords.
     */ 
    static int getCount() {

        return getInteger("password.count", DEFAULT_COUNT); 
    }

    /**
     * We return the minimal length of passwords.
     * @return the minimal length of passwords.
     */ 
    static int getMinLength() {

        return getInteger("password.length.minimum", DEFAULT_MIN_LENGTH); 
    }

    /**
     * We return the maximal length of passwords.
     * @return the maximal length of passwords.
     */ 
    static int getMaxLength() {

        return getInteger("password.length.maximum", DEFAULT_MAX_LENGTH); 
    }

    /**
     * We return the initial mask of passwords.
     * @return the initial mask of passwords.
     */ 
    static int getInitialMask() {

        return getInteger("initial.mask", RandomPassword.ALLS); 
    }

    /**
     * We return the initial length of passwords.
     * @return the initial length of passwords.
     */ 
    static int getInitialLength() {

        return getInteger("initial.length", DEFAULT_MAX_LENGTH); 
    }

    /**
     * We return the ressource bundle.
     * @return the ressource bundle.
     */ 
    static ResourceBundle getRB() {

        return Parameters.getInstance()._rb;
    }

    /**
     * We return an integer value.
     *
     * @param ids the string identifier.
     * @param val the default value.
     * @return a integer value 
     */ 
    private static int getInteger(String ids, int val) {

        int ret = val;

        try {
            String strTmp = getRB().getString(ids);
            Integer intTmp = Integer.decode(strTmp);
            if (intTmp != null) {
                ret = intTmp.intValue();
            }
        } catch (MissingResourceException e) {
             System.err.println("** MissingResourceException:" 
                     + e.getMessage());
        } catch (NumberFormatException e) {
             System.err.println("** NumberFormatException(" + ids + "):" 
                     + e.getMessage());
        } catch (Exception e) {
             System.err.println("** Exception:" 
                     + e.getMessage());
        }

        return ret; 
    }

} // End of Parameters.java

